import os, re, time
import yaml
import paramiko
import flask, json
from flask import request, Response
from flask_cors import *

# 请求接口
SERVER = flask.Flask(__name__)
CORS(SERVER, resources=r'/*')

# 读取yaml
def refresh_ctx(path: str, encoding='utf-8'):
    with open(path, "r", encoding = encoding) as yml:
        content = list(yaml.load_all(yml.read(), yaml.Loader))
    return content[0] if len(content) == 1 else content

# 写入yaml
def write_ctx(path: str, content):
    with open(path, "w", encoding="utf-8") as file:
        yaml.dump(content, file, indent=2, sort_keys=False, allow_unicode=True)
    pass

# 返回请求数据
def responseData(data, message = ''):

	res = ''
	if data is not None: res = {'code': 200, 'data': data, 'message': '请求成功'}
	else: res = {'code': 500, 'data': {}, 'message': message}
	return Response(json.dumps(res), mimetype='application/json')

# 发送命令
def sendCommand(ipaddr, command, scp = False):

	pkey = paramiko.RSAKey.from_private_key_file('/root/.ssh/hkqtkey.pem')
	# pkey = paramiko.RSAKey.from_private_key_file('./hkqtkey.pem')
	trans = paramiko.Transport((ipaddr, 22))
	trans.connect(username = 'root', pkey = pkey)
	
	ssh = paramiko.SSHClient()
	ssh._transport = trans
	stdin, stdout, stderr = ssh.exec_command(command)
	data = stdout.read().decode().split()
	trans.close()
	return data
	

# 获取所有平台信息
@SERVER.route('/getNameSpace', methods=['get', 'post'])
def getPlatforms():
	return responseData(refresh_ctx('param.yaml'))

# 获取应用
@SERVER.route('/getPods', methods=['get', 'post'])
def getPods():

	platforms = refresh_ctx('param.yaml')
	namespace = request.values.get('ns')
	if namespace is None:
		return responseData(None, "应用别名不可为空")

	if namespace not in platforms.keys():
		return responseData(None, "不存在当前平台")

	if platforms[namespace]['type'] == 'k8s':
		command = f"kubectl get pods -n {namespace} -o=name" + " | awk -F '/' '{print $NF}'"
		pods = sendCommand(platforms[namespace]['ips'][0], command)
		platforms[namespace]['pods'] = pods
		write_ctx('./param.yaml', platforms)

		pods = list(map(lambda x: re.search(r"([\w-]+)-\w+-\w+", x).group(1), pods))
		pods = list(set(pods))
		data = []
		for pod in pods:
			data.append({"ip": platforms[namespace]['ips'][0], "pod": pod})
		return responseData(data)
	return responseData({'cmd': command, 'list': []})

# 开始抓包
@SERVER.route('/packetCapture', methods=['get', 'post'])
def packetCapture():

	namespace = request.values.get('ns')
	pod = request.values.get('pod')
	peerId = request.values.get('ipaddr')

	platforms = refresh_ctx('param.yaml')
	remoteIpaddr = platforms[namespace]['ips'][0]	
	pods = platforms[namespace]['pods']
	if pod in pods:
		return responseData(None, "应用名称不存在")

	pod = [name for name in pods if re.search(f'^{pod}-', name)][0]
	ipaddrCmd = ''
	packetName = f'{namespace}-{pod}.pcap'
	if peerId != '' and peerId is not None:
		ipaddrCmd = f'\"-f host {peerId}\"'
		packetName = f'{namespace}-{pod}-{remoteIpaddr}.pcap'

	# 开始抓包
	cmd = f"nohup kubectl sniff {pod} -n {namespace} " + f"{ipaddrCmd} " + f"-o /pcap/{packetName} > /dev/null 2>&1 &"
	sendCommand(remoteIpaddr, cmd)

	# 获取pid返回
	time.sleep(0.5)
	pidCmd = f'ps -ef | grep {packetName} ' + '| awk \'{print $2}\''
	pid = sendCommand(remoteIpaddr, pidCmd)[0]

	# 地址
	url = f'http://154.92.108.28/{packetName}'
	data = {'pid': pid, 'filePath': url}
	# 写入进程列表
	write_ctx('./process.yaml', {pid: remoteIpaddr})
	return responseData(data)

# 停止抓包
@SERVER.route('/stopCapture', methods=['get', 'post'])
def stopCapture():

	process = refresh_ctx('process.yaml')
	pid = request.values.get('pid')
	ipaddr = request.values.get('ip')
	if not pid:
		return responseData(None, "PID不可为空")

	# 拷贝pcap文件
	if ipaddr == '8.218.111.151':
		cmd = f'scp -i /root/.ssh/hkqtkey.pem root@{ipaddr}:/pcap/* /pcap/'
		sendCommand("8.210.1.22", cmd)

	# 停止进程
	cmd = f"export name=$(ps -ef | grep {pid} | grep pcap" + " | awk '{print $NF}');" + f" kill -9 {pid}; "
	if ipaddr == '8.218.111.151':
		cmd += "rm -rf ${name}"
	sendCommand(ipaddr, cmd)

	# 更新进程表
	del process[str(pid)]
	write_ctx('./process.yaml', process)
	return responseData({'cmd': cmd})

if __name__ == '__main__':

	SERVER.run(debug=True, port=8088, host='0.0.0.0')
